@@ -1,6 +1,8 @@
 package Net::MAC::Vendor;
 use strict;
 
+use v5.10;
+
 =encoding utf8
 
 =head1 NAME
@@ -32,19 +34,25 @@ This module allows you to take a MAC address and turn it into the OUI
 and vendor information. You can, for instance, scan a network,
 collect MAC addresses, and turn those addresses into vendors. With
 vendor information, you can often guess at what what you are looking
-at (e.g. an Apple product).
+at (I<e.g.> an Apple product).
 
 You can use this as a module as its individual functions, or call it
 as a script with a list of MAC addresses as arguments. The module can
 figure it out.
 
-This module tries to persistently cache with DBM::Deep the OUI
-information so it can avoid using the network. If it cannot load
-DBM::Deep, it uses a normal hash (which is lost when the process
-finishes). You can preload this cache with the C<load_cache()> function.
-So far, the module looks in the current working directory for a file
-named mac_oui.db to find the cache. I need to come up with a way to
-let the user set that location.
+The IEEE moves the location of its OUI file. If they do that again, you
+can set the C<NET_MAC_VENDER_OUI_URL> environment variable to get the new
+URL without updating the code.
+
+Here are some of the old URLs, which also flip-flop schemes:
+
+	http://standards.ieee.org/regauth/oui/oui.txt
+	https://standards.ieee.org/regauth/oui/oui.txt
+	http://standards-oui.ieee.org/oui.txt
+
+There are older copies of the OUI file in the GitHub repository.
+
+These files are large (about 4MB), so you might want to cache a copy.
 
 =head2 Functions
 
@@ -52,18 +60,15 @@ let the user set that location.
 
 =cut
 
-use base qw(Exporter);
+use Exporter qw(import);
 
 __PACKAGE__->run( @ARGV ) unless caller;
 
 use Carp;
-use LWP::Simple qw(get);
+use Mojo::URL;
+use Mojo::UserAgent;
 
-# https://standards.ieee.org/regauth/oui/oui.txt
-
-our $Cached = {};
-
-our $VERSION = '1.23';
+our $VERSION = '1.26';
 
 =item run( @macs )
 
@@ -95,6 +100,17 @@ sub run {
 	return 1;
 	}
 
+=item ua
+
+Return the Mojo::UserAgent object used to fetch resources.
+
+=cut
+
+sub ua {
+	state $ua = Mojo::UserAgent->new->max_redirects(3);
+	$ua;
+	}
+
 =item lookup( MAC )
 
 Given the MAC address, return an anonymous array with the vendor
@@ -102,6 +118,9 @@ information. The first element is the vendor name, and the remaining
 elements are the address lines. Different records may have different
 numbers of lines, although the first two should be consistent.
 
+This makes a direct request to the IEEE website for that OUI to return
+the information for that vendor.
+
 The C<normalize_mac()> function explains the possible formats
 for MAC.
 
@@ -178,10 +197,43 @@ To avoid multiple calls on the network, use C<load_cache> to preload
 the entire OUI space into an in-memory cache. This can take a long
 time over a slow network, though; the file is about 60,000 lines.
 
+Also, the IEEE website has been flaky lately, so loading the cache is
+better. This distribution comes with several versions of the complete
+OUI data file.
+
 =cut
 
 sub fetch_oui {
-	fetch_oui_from_cache( $_[0] ) || fetch_oui_from_ieee( $_[0] );
+	# fetch_oui_from_custom( $_[0] )    ||
+		fetch_oui_from_cache( $_[0] ) ||
+		fetch_oui_from_ieee( $_[0] );
+	}
+
+=item fetch_oui_from_custom( MAC, [ URL ] )
+
+Looks up the OUI information from the specified URL or the URL set
+in the C<NET_MAC_VENDOR_OUI_SOURCE> environment variable.
+
+The C<normalize_mac()> function explains the possible formants for
+MAC.
+
+=cut
+
+sub fetch_oui_from_custom {
+	my $mac = normalize_mac( shift );
+	my $url = shift // $ENV{NET_MAC_VENDOR_OUI_SOURCE};
+
+	return unless defined $url;
+
+	my $html = get( $url );
+	unless( defined $html ) {
+		carp "Could not fetch data from the IEEE!";
+		return;
+		}
+
+	parse_oui(
+		extract_oui_from_html( $html, $mac )
+		);
 	}
 
 =item fetch_oui_from_ieee( MAC )
@@ -194,10 +246,31 @@ MAC.
 
 =cut
 
+sub _search_url_base {
+	state $url = Mojo::URL->new(
+		'http://standards.ieee.org/cgi-bin/ouisearch'
+		);
+	$url;
+	}
+
+sub _search_url {
+	my( $class, $mac ) = @_;
+	my $url = $class->_search_url_base->clone;
+	$url->query( $mac );
+	}
+
 sub fetch_oui_from_ieee {
 	my $mac = normalize_mac( shift );
 
-	my $html = get( "https://standards.ieee.org/cgi-bin/ouisearch?$mac" );
+	my @urls = __PACKAGE__->_search_url( $mac );
+
+	my $html;
+	URL: foreach my $url ( @urls ) {
+		$html = __PACKAGE__->_fetch_oui_from_url( $url );
+		next URL unless defined $html;
+		last;
+		}
+
 	unless( defined $html ) {
 		carp "Could not fetch data from the IEEE!";
 		return;
@@ -208,6 +281,35 @@ sub fetch_oui_from_ieee {
 		);
 	}
 
+sub _fetch_oui_from_url {
+	my( $class, $url ) = @_;
+	my $tries = 0;
+
+	return unless defined $url;
+
+	TRY: {
+		my $tx = __PACKAGE__->ua->get( $url );
+		unless( $tx->success ) {
+			if( $tries > 3 ) {
+				carp "Failed fetching [$url]: " . $tx->res->code;
+				return;
+				}
+
+			$tries++;
+			sleep 1 * $tries;
+			redo TRY;
+			}
+
+		my $html = $tx->res->body;
+		unless( defined $html ) {
+			carp "No content in response for [$url]!";
+			return;
+			}
+
+		return $html;
+		}
+	}
+
 =item fetch_oui_from_cache( MAC )
 
 Looks up the OUI information in the cached OUI information (see
@@ -226,7 +328,7 @@ If it doesn't find the MAC in the cache, it returns nothing.
 sub fetch_oui_from_cache {
 	my $mac = normalize_mac( shift );
 
-	exists $Cached->{ $mac } ? $Cached->{ $mac } : ();
+	__PACKAGE__->get_from_cache( $mac );
 	}
 
 =item extract_oui_from_html( HTML, OUI )
@@ -244,31 +346,11 @@ sub extract_oui_from_html {
 	my $html = shift;
 	my $lookup_mac = normalize_mac( shift );
 
-	$html =~ s/<pre>.*?$lookup_mac/<pre>$lookup_mac/is;
+	my( $record ) = $html =~ m|<pre>(<b>$lookup_mac</b>.*?)</pre>|is;
+	$record =~ s|</?b>||g;
 
-	# sometimes the HTML returns more than one OUI because
-	# IEEE has a problem parsing their own data when they
-	# have private blocks
-	my( $ouis ) = $html =~ m|<pre>(.*?)</pre>|gs;
-	return unless defined $ouis;
-	$ouis =~ s/<\/?b>//gs; # remove bold around the OUI
-
-	my @entries = split /\n\s*\n/, $ouis;
-	return unless defined $entries[0];
-	return $entries[0] unless defined $entries[1];
-
-	my $result = $entries[0];
-
-	foreach my $entry ( @entries ) {
-		$entry =~ s/^\s+|\s+$//;
-		my $found_mac = normalize_mac( substr $entry, 0, 8 );
-		if( $found_mac eq $lookup_mac ) {
-			$result = $entry;
-			last;
-			}
-		}
-
-	return $result;
+	return unless defined $record;
+	return $record;
 	}
 
 =item parse_oui( STRING )
@@ -292,7 +374,7 @@ With no arguments, it returns an empty anonymous array.
 sub parse_oui {
 	my $oui = shift;
 	return [] unless $oui;
-
+	$oui =~ s|</?b>||g;
 	my @lines = map { s/^\s+//; $_ ? $_ : () } split /$/m, $oui;
 	splice @lines, 1, 1, ();
 
@@ -300,13 +382,34 @@ sub parse_oui {
 	return \@lines;
 	}
 
+=item oui_url
+
+=item oui_urls
+
+Returns the URLs of the oui.txt resource. The IEEE likes to move this
+around. These are the default URL that C<load_cache> will use, but you
+can also supply your own with the C<NET_MAC_VENDOR_OUI_URL> environment
+variable.
+
+=cut
+
+sub oui_url { (grep { /\Ahttp:/ } &oui_urls)[0] }
+
+sub oui_urls {
+	my @urls = 'http://standards-oui.ieee.org/oui.txt';
+
+	unshift @urls, $ENV{NET_MAC_VENDOR_OUI_URL}
+		if defined $ENV{NET_MAC_VENDOR_OUI_URL};
+
+	@urls;
+	}
+
 =item load_cache( [ SOURCE[, DEST ] ] )
 
-Downloads the current list of all OUIs, parses it with C<parse_oui()>,
-and stores it in C<$Cached> anonymous hash keyed by the OUIs (i.e.
-00-0D-93). The C<fetch_oui()> will use this cache if it exists.
+Downloads the current list of all OUIs in SOURCE, parses it with C<parse_oui()>,
+and stores it in the cache. The C<fetch_oui()> will use this cache if it exists.
 
-By default, this uses C<https://standards.ieee.org/regauth/oui/oui.txt>, 
+By default, this uses the URL from C<oui_url>,
 but given an argument, it tries to use that. To load from a local
 file, use the C<file://> scheme.
 
@@ -314,7 +417,7 @@ If C<load_cache> cannot load the data, it issues a warning and returns
 nothing.
 
 This previously used DBM::Deep if it was installed, but that was much
-too slow. Instead, if you want persistence, you can play with 
+too slow. Instead, if you want persistence, you can play with
 C<$Net::MAC::Vendor::Cached> yourself.
 
 If you want to store the data fetched for later use, add a destination
@@ -324,50 +427,51 @@ specify C<undef> as source.
 =cut
 
 sub load_cache {
-	my $source = shift || "https://standards.ieee.org/regauth/oui/oui.txt";
-	my $dest   = shift;
+	my( $source, $dest ) = @_;
 
-	my $data = do {
-		if( -e $source ) { # local files
-			do { local( @ARGV, $/ ) = $source; <> }
-			}
-		else { # everything else
-			my $data = get( $source );
-
-			unless( defined $data ) {
-				carp "Could not read from '$source'";
+	my $data = do {;
+		if( defined $source ) {
+			unless( -e $source ) {
+				carp "Net::Mac::Vendor cache source [$source] does not exist";
 				return;
 				}
 
-			if ( $dest ) { # store cache
-				if ( open my $fh, '>', $dest ) {
-					print $fh $data;
-					close $fh;
-					} 
-				else { # notify on error, but continue
-					carp "Could not write to '$dest'";
-					}
-				}
-
-			$data;
+			do { local( @ARGV, $/ ) = $source; <> }
+			}
+		else {
+			#say time . " Fetching URL";
+			my $tx = __PACKAGE__->ua->get( oui_url() );
+			#say time . " Fetched URL";
+			#say "size is " . $tx->res->headers->header( 'content-length' );
+			$tx->res->body;
 			}
 		};
 
+	if( defined $dest ) {
+		if( open my $fh, '>:utf8', $dest ) {
+			print { $fh } $data;
+			close $fh;
+			}
+		else { # notify on error, but continue
+			carp "Could not write to '$dest': $!";
+			}
+		}
+
+
 	# The PRIVATE entries fill in a template with no
 	# company name or address, but the whitespace is
 	# still there. We need to split on a newline
 	# followed by some potentially horizontal whitespace
 	# and another newline
-	my @entries = split /[\t ]*\n[\t ]*\n/, $data;
+	my $CRLF = qr/(?:\r?\n)/;
+	my @entries = split /[\t ]* $CRLF [\t ]* $CRLF/x, $data;
 	shift @entries;
 
 	my $count = '';
 	foreach my $entry ( @entries ) {
-	#	print STDERR "Processing ", ++$count, " entries\n";
 		$entry =~ s/^\s+//;
 		my $oui = substr $entry, 0, 8;
-		$Cached->{ $oui } = parse_oui( $entry );
-		#last if $count > 100;
+		__PACKAGE__->add_to_cache( parse_oui( $entry ) );
 		}
 
 	return 1;
@@ -375,6 +479,56 @@ sub load_cache {
 
 =back
 
+=head1 Caching
+
+Eventually I want people to write their own caching classes so I've
+created some class methods for this.
+
+=over 4
+
+=cut
+
+BEGIN {
+my $Cached = {};
+
+=item add_to_cache
+
+Add to the cache. This is mostly in place for a future expansion to
+full objects so you can override this in a subclass.
+
+=cut
+
+sub add_to_cache {
+	my( $class, $oui, $parsed ) = @_;
+
+	$Cached->{ $oui } = $parsed;
+	}
+
+=item get_from_cache
+
+Get from the cache. This is mostly in place for a future expansion to
+full objects so you can override this in a subclass.
+
+=cut
+
+sub get_from_cache {
+	my( $class, $oui ) = @_;
+
+	$Cached->{ $oui };
+	}
+
+=item get_cache_hash
+
+Get the hash the built-in cache uses. You should only use this if you
+were using the old C<$Cached> package variable.
+
+=cut
+
+sub get_cache_hash { $Cached }
+}
+
+=back
+
 =head1 SEE ALSO
 
 L<Net::MacMap>
@@ -391,7 +545,7 @@ brian d foy C<< <bdfoy@cpan.org> >>
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (c) 2004-2014 brian d foy.  All rights reserved.
+Copyright © 2004-2015, brian d foy C<< <bdfoy@cpan.org> >>. All rights reserved.
 
 This program is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself.
